﻿using System;
using System.Data;
using System.Text;
using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using System.Collections.Generic;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using System.Collections.ObjectModel;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Shamrock.Providers;
using Produmex.Foundation.Collections;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_RepackScriptShamrock
{
    /// <summary>
    /// Shamrock
    /// </summary>
    public class WorkflowScript_RepackScriptShamrock : WorkflowInstanceScriptBase
    {
        /// <summary>
        /// Definitions
        /// </summary>
        public static class Definitions
        {
      
        }

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        ISboProviderService m_sboProviderService;

        // Output parameters

        // Sub-flows
        WorkflowInstanceBase m_enterQuantitySubFlow;
        // Sub control
        protected bool SubBackRequested;

        //// Sub values
        protected double SubQuantity;
        protected double? SubQuantityUom2;



        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_RepackScriptShamrock"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_RepackScriptShamrock(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            m_enterQuantitySubFlow = this.Create("EnterQuantityScript");
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            
            Message msg = null;
            string initialErrorKey = null;
            string deviceID = session.DeviceInstanceId;
            string query = null;
            string scannedSSCC = null;
            string masterSSCC = null;            
            string locationCode = null;
            string locationCodeChild = null;
            string itemCode = null;
            string qualityStatusCode = null;
            string batchNumber = null;
            string[] buttonKeys = null;
           
            int? luid = null;
            int? masterLUID = null;
            int? itemTransactionalInfoKey = null;

            DataSet ds = null;
            DataSet dsSSCC = null;
            DataSet dsItems = null;
            DataSet dsBatchNumbers = null;
            DataSet dsLocations = null;
            DataSet dsProductsOnSSCC = null;
        
            DateTime? bestBeforeDate = null;
           
            PmxItemInfo itemInfo = null;
            PmxOseWarehouseInfo whsInfo = null;
            BarcodeContainer barcodeContainerLUID;

            bool fromStep_IdentifySSCCOrLocation = false;
            bool from_ValidateSSCC = false;
            bool goBack = false;        

            double batchQuantity = 0;
            double qtyRepack= 0.0;
            double? qtyUom2Repack;
            double qtyChild = 0.0;
            double? qtyUom2Child;
            double initialsQty;         

                      
            whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);
            
            SboUserInfo userInfo = m_sboProviderService.GetCurrentUser();

            IBarcodeController barcodeController = null;

            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
            delegate (PmxDbConnection conn, object[] parameters)
            {
                PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
                barcodeController = bcProv.GetBarcodeController();


                return null;
            });

            Step_IdentifySSCCOrLocation:

            #region IDENTIFY SSCC OR LOCATION

            goBack = false;
            buttonKeys = new string[1] { "MSG_BUTTON_SELECT_LOCATION" };


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SCAN_SSCC_OR_LOCATION",
                "BarcodeController", barcodeController,
                "AllowedButtons", buttonKeys,
                "ForceGS1_128", false
                ));

            msg = WaitForMessage();

            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".DataEntered"))
            {
                 barcodeContainerLUID = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

                //check what has been scanned
                if (barcodeContainerLUID.GS1DataObject != null && !string.IsNullOrEmpty(barcodeContainerLUID.GS1DataObject.Sscc))
                {
                    //user scanned an SSCC, so we check if it is allowed
                    scannedSSCC = barcodeContainerLUID.GS1DataObject.Sscc;
                    goto Step_CheckSSCC;
                }
                else
                {
                    scannedSSCC = barcodeContainerLUID.FreeBarcode;

                    if (scannedSSCC != null && scannedSSCC.Length == 20 && scannedSSCC.StartsWith("00"))
                    {
                        scannedSSCC = scannedSSCC.Substring(2);
                        goto Step_CheckSSCC;
                    }
                }

                string tempLocation = BarcodeHelper.GetProductIdentification(barcodeContainerLUID);

              
                if (string.IsNullOrEmpty(tempLocation))
                {
                    initialErrorKey = "MSG_ERROR_BARCODE_DOES_NOT_CONTAIN_SSCC_OR_LOCATION";
                    goto Step_IdentifySSCCOrLocation;
                }
                else
                {
                    //user scanned a location
                    locationCode = tempLocation;
                    fromStep_IdentifySSCCOrLocation = true;
                    goto Step_CheckLocation;
                   
                }
            }

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                fromStep_IdentifySSCCOrLocation = true;
                goto Step_SelectLocation;
            }

            //Go back
            return;

            #endregion
            
            Step_CheckLocation:

            #region CHECK LOCATION

            //Check if there are multiple SSCC's on the location
            query = BuildQuery.GetSSCCForLocation(locationCode);
            dsSSCC = m_sboProviderService.RunView(true, null, null, query);

            if (dsSSCC.Tables[0].Rows.Count > 1)
            {   
                //location has multiple sscc           
                goto Step_IdentifySSCC;
            }

            if (dsSSCC.Tables[0].Rows.Count == 1)
            {
                luid = Convert.ToInt32(dsSSCC.Tables[0].Rows[0][SSCCDataSet.Columns.LUID.Name]);
                scannedSSCC = Convert.ToString(dsSSCC.Tables[0].Rows[0][SSCCDataSet.Columns.SSCC.Name]);
                goto Step_CheckSSCC;
            }

            locationCode = null;
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                   "MessageKey", "MSG_ERROR_INCORRECT_LOCATION",
                       "NoTranslationOfMessageKey", false,
                       "ShowButton", true));
            WaitForMessage();

            goto Step_IdentifySSCCOrLocation;

            #endregion

            Step_IdentifySSCC:

            #region IDENTIFY SSCC


            bool showButtonNoSSCC = false;

            if (this.GetNumberOfRows(dsSSCC) > 0)
            {
                foreach (DataRow ssccRow in dsSSCC.Tables[0].Rows)
                {
                    if (string.IsNullOrEmpty(ssccRow[SSCCDataSet.Columns.SSCC.Name] as string))
                    {
                        showButtonNoSSCC = true;

                        //remove current row
                        dsSSCC.Tables[0].Rows.Remove(ssccRow);
                        dsSSCC.AcceptChanges();

                        break;
                    }
                }
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectSSCCScreen),
                     DefaultCultureInfo.Get(), BuildParamCollection(
                         "InitialErrorKey", initialErrorKey,
                         "TitleKey", "MSG_TITLE_SELECT_SSCC",
                         "SSCCDS", dsSSCC,
                         "ShowButtonNoSSCC", showButtonNoSSCC
                         ));
            msg = WaitForMessage();

            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".SSCCSelected"))
            {
                luid = ExtractParameter<int?>(msg.Parameters, "luid");
                scannedSSCC = ExtractParameter<string>(msg.Parameters, "sscc");
                goto Step_CheckSSCC;

            }

            if (msg.Name.EndsWith(".NoSSCC"))
            {
                luid = null;
                scannedSSCC = null;
                goto Step_IdentifyProduct;
            }
            
            goto Step_IdentifySSCCOrLocation;

            #endregion

            Step_CheckSSCC:

            #region CHECK SSCC

            //Check if the SSCC that was scanned exists in stock
            query = BuildQuery.GetSSCCInfo(scannedSSCC, null);
            ds = m_sboProviderService.RunView(false, null, null, query);
            if (GetNumberOfRows(ds) > 0)
            {

                string tempLocationCode = ds.Tables[0].Rows[0][SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name] as string;
                luid= ds.Tables[0].Rows[0][SSCCDataSet.Columns.LUID.Name] as int?;                 
                locationCode = tempLocationCode;

                if (whsInfo == null)
                {
                    whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), tempLocationCode);
                }
                else
                {
                    PmxOseWarehouseInfo tempWhsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), tempLocationCode);
                    if (tempWhsInfo.PmxWarehouseCode != whsInfo.PmxWarehouseCode)
                    {
                        //The SSCC is not in the same whs
                        initialErrorKey = "MSG_ERROR_SSCC_IN_DIFFERENT_WHS";
                        if (fromStep_IdentifySSCCOrLocation)
                        {
                            goto Step_IdentifySSCCOrLocation;
                        }
                        else
                        {
                            goto Step_IdentifyProduct;
                        }
                    }
                }

                //check if location is locked
                if (m_sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteIsLocationLocked), tempLocationCode))
                {
                    //the location is locked
                    initialErrorKey = "MSG_ERROR_LOCATION_LOCKED";
                    if (fromStep_IdentifySSCCOrLocation)
                    {
                        goto Step_IdentifySSCCOrLocation;
                    }
                    else
                    {
                        goto Step_IdentifyProduct;
                    }
                }

               
                goto Step_IdentifyProduct;

            }
            else
            {
                //SSCC does not exist in stock
                initialErrorKey = "MSG_ERROR_SSCC_NOT_IN_STOCK";
                goto Step_IdentifySSCCOrLocation;
            }        

            #endregion

            Step_IdentifyProduct:

            #region SELECT PRODUCT

            batchNumber = null;
            bestBeforeDate = null;
            itemTransactionalInfoKey = null;
            qualityStatusCode = null;
            batchQuantity = 0;

            if (goBack)
            {
                goto Step_IdentifySSCCOrLocation;
            }



            query = BuildQuery.GetProductsFromStockOrderByName(locationCode, scannedSSCC, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Inventory, true, m_sboProviderService.GetDbTool());
            dsItems = m_sboProviderService.RunView(true, null, null, query);

            if (this.GetNumberOfRows(dsItems) == 0)
            {
                initialErrorKey = "MSG_ERROR_PRODUCT_NOT_FOUND";
                goto Step_IdentifySSCCOrLocation;
            }

            if (this.GetNumberOfRows(dsItems) == 1)
            {
                //there is only one item, so auto select it!
                DataRow row = dsItems.Tables[0].Rows[0];
                itemCode = row[ProductDataSet.Columns.PRODUCT_CODE.Name] as string;
              //  luid = Convert.ToInt16(row[PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]);
                //select the batch
                goto Step_SelectBatchNumber;
                
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_PRODUCT",
                    "ProductDS", dsItems
                    ));
            msg = WaitForMessage();

            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".ProductSelected"))
            {
                itemCode = ExtractParameter<string>(msg.Parameters, "itemCode");
                //select the batch
                goto Step_SelectBatchNumber;
            }

            goto Step_IdentifySSCCOrLocation;


            #endregion

            Step_SelectBatchNumber:

            #region SELECT BATCHNUMBER

            batchNumber = null;
            bestBeforeDate = null;
            itemTransactionalInfoKey = null;
            qualityStatusCode = null;
            batchQuantity = 0;

            itemInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCode);

            //if (!itemInfo.HasBatchnumber)
            //{
            //    batchQuantity
            //        goto Step_EnterQuantityRepack;
            // }

                query = BuildQuery.GetBatchNumbersFromStock(scannedSSCC, locationCode, itemCode, null, null, null, false, true);
            dsBatchNumbers = m_sboProviderService.RunView(true, null, null, query);

           

                if (this.GetNumberOfRows(dsBatchNumbers) == 0)
                {
                
                    initialErrorKey = "MSG_ERROR_PRODUCT_BATCHNUMBER_NOT_FOUND";
                    goto Step_IdentifyProduct;
                }
               
          

                if (this.GetNumberOfRows(dsBatchNumbers) == 1)
                {
                    DataRow row = dsBatchNumbers.Tables[0].Rows[0];
                    batchNumber = row[BatchNumberDataSet.Columns.BATCH_NUMBER.Name] as string;
                    batchQuantity = Convert.ToDouble(row[BatchNumberDataSet.Columns.QUANTITY.Name]);
                    qualityStatusCode = row[BatchNumberDataSet.Columns.QUALITY_STATUS_CODE.Name] as string;
                    if (row[BatchNumberDataSet.Columns.BEST_BEFORE_DATE.Name] is DateTime)
                    {
                        bestBeforeDate = Convert.ToDateTime(row[BatchNumberDataSet.Columns.BEST_BEFORE_DATE.Name]);
                    }
                    else
                    {
                        bestBeforeDate = null;
                    }
                    itemTransactionalInfoKey = Convertor.ConvertToInt32Nullable(row[BatchNumberDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name]);

                    goto Step_EnterQuantityRepack;
                }


                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectBatchNumberScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_BATCH_NUMBER",
                        "BatchNumberDS", dsBatchNumbers
                        ));
                msg = WaitForMessage();

                //clear data
                initialErrorKey = null;

                if (msg.Name.EndsWith(".BatchNumberSelected"))
                {
                    batchNumber = ExtractParameter<string>(msg.Parameters, "batchNumber");
                    bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "bestBeforeDate");
                    batchQuantity = ExtractParameter<double>(msg.Parameters, "quantity");
                    qualityStatusCode = ExtractParameter<string>(msg.Parameters, "qualityStatusCode");
                    itemTransactionalInfoKey = ExtractParameter<int?>(msg.Parameters, "itemTransactionalInfoKey");

                    goto Step_EnterQuantityRepack;
                }

          

            goto Step_IdentifyProduct;

            #endregion

            Step_EnterQuantityRepack:

            #region ENTER QUANTITY REPACK

            SubQuantity = 0.0;
            SubQuantityUom2 = null;
            SubBackRequested = false;

            m_enterQuantitySubFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY_REPACK"),
                new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfo),
                new ReadOnlyBinder<int?>("ItriKey", itemTransactionalInfoKey),
                new ReadOnlyBinder<string>("BatchNumber", batchNumber),
                new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
                new ReadOnlyBinder<string>("SSCC", scannedSSCC ),

                new ReadOnlyBinder<bool>("AllowZero", false),
                new ReadOnlyBinder<double>("InitialQuantity", 0.0),
                new ReadOnlyBinder<double>("MaximumQuantity", batchQuantity),
                new ReadOnlyBinder<string>("UOM", itemInfo.InventoryUom),
                new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfo == null) ? 0 : itemInfo.UomDecimals),
                new ReadOnlyBinder<double>("InnerQuantity", 0.0),
                new ReadOnlyBinder<string>("InnerUOM", null),

                new ReadOnlyBinder<string>("DocNum", null),
                new ReadOnlyBinder<double>("StillToQuantity", batchQuantity),
                new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_CURRENT_STOCK"),
                new ReadOnlyBinder<string>("MessageKeyStillQuantityWeight", null),
                new ReadOnlyBinder<string>("MessageKeyBatchnumber", null),

                new ReadOnlyBinder<bool>("ForceInputSerialNumbers", false),
                new ReadOnlyBinder<string>("ObjectType", Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxMove),
                new ReadOnlyBinder<int?>("DocEntry", null),
                new ReadOnlyBinder<bool>("UseSerialNumberRange", false),
                new ReadOnlyBinder<string>("LocationCode", locationCode ),

                new ReadOnlyBinder<int?>("LUID", luid),
                new ReadOnlyBinder<BarcodeContainer>("ScannedBarcodeContainer", null),

                new ReadWriteBinder<bool>("BackRequested", this, "SubBackRequested"),
                new ReadWriteBinder<double>("QuantityEntered", this, "SubQuantity"),
                new ReadWriteBinder<double?>("QuantityUom2Entered", this, "SubQuantityUom2"),
                new ReadOnlyBinder<bool>("SetInitialQuantity", true)
                );

            base.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

            if (SubBackRequested)
            {
                goBack = true;
                goto Step_IdentifyProduct;
            }

            qtyRepack = SubQuantity;
            qtyUom2Repack = SubQuantityUom2;


            goto Step_EnterQuantityChild;

            #endregion

            Step_EnterQuantityChild:

            #region ENTER QUANTITY CHILD SSCC

            SubQuantity = 0.0;
            SubQuantityUom2 = null;
            initialsQty = 1.0;

            SubBackRequested = false;
            m_enterQuantitySubFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY_CHILD"),
                new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfo),
                new ReadOnlyBinder<int?>("ItriKey", itemTransactionalInfoKey),
                new ReadOnlyBinder<string>("BatchNumber", batchNumber),
                new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
                new ReadOnlyBinder<string>("SSCC", scannedSSCC),

                new ReadOnlyBinder<bool>("AllowZero", false),
                new ReadOnlyBinder<double>("InitialQuantity", initialsQty ),
                new ReadOnlyBinder<double>("MaximumQuantity", -1),
                new ReadOnlyBinder<string>("UOM", itemInfo.InventoryUom),
                new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfo == null) ? 0 : itemInfo.UomDecimals),
                new ReadOnlyBinder<double>("InnerQuantity", 0.0),
                new ReadOnlyBinder<string>("InnerUOM", null),

                new ReadOnlyBinder<string>("DocNum", null),
                new ReadOnlyBinder<double>("StillToQuantity", -1),
                new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_CURRENT_STOCK"),
                new ReadOnlyBinder<string>("MessageKeyStillQuantityWeight", null),
                new ReadOnlyBinder<string>("MessageKeyBatchnumber", null),

                new ReadOnlyBinder<bool>("ForceInputSerialNumbers", false),
                new ReadOnlyBinder<string>("ObjectType", Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxMove),
                new ReadOnlyBinder<int?>("DocEntry", null),
                new ReadOnlyBinder<bool>("UseSerialNumberRange", false),
                new ReadOnlyBinder<string>("LocationCode", locationCode),

                new ReadOnlyBinder<int?>("LUID", null),
                new ReadOnlyBinder<BarcodeContainer>("ScannedBarcodeContainer", null),

                new ReadWriteBinder<bool>("BackRequested", this, "SubBackRequested"),
                new ReadWriteBinder<double>("QuantityEntered", this, "SubQuantity"),
                new ReadWriteBinder<double?>("QuantityUom2Entered", this, "SubQuantityUom2"),
                new ReadOnlyBinder<bool>("SetInitialQuantity", true)
                );

            base.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

            if (SubBackRequested)
            {
                goBack = true;
                goto Step_IdentifyProduct;
            }


            qtyChild = SubQuantity;
            qtyUom2Child = SubQuantityUom2;

            goto Step_CheckChildinput;

          

            #endregion


            Step_CheckChildinput:

            #region CHECK CHILD INPUT

            bool correctQty = ( (qtyRepack / qtyChild) == Math.Round((qtyRepack / qtyChild),itemInfo.UomDecimals));
             
            if (!correctQty)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                   "MessageKey", "MSG_ERROR_INCORRECT_QUANTITY",
                       "NoTranslationOfMessageKey", false,
                       "ShowButton", true));
                WaitForMessage();
                goto Step_EnterQuantityChild;
            }
            else
            {
                goto Step_EnterDestinationLocation;
            }
            #endregion


            Step_EnterDestinationLocation:

            #region ENTER DESTINATION LOCATION
                        
  
            buttonKeys = new string[1] { "MSG_BUTTON_SELECT_LOCATION" };


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SCAN_SSCC_OR_LOCATION",
                "BarcodeController", barcodeController,
                "AllowedButtons", buttonKeys,
                "ForceGS1_128", false
                ));

            msg = WaitForMessage();

      
            if (msg.Name.EndsWith(".DataEntered"))
            {
                barcodeContainerLUID = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

                //check what has been scanned
                if (barcodeContainerLUID.GS1DataObject != null && !string.IsNullOrEmpty(barcodeContainerLUID.GS1DataObject.Sscc))
                {
                    //user scanned an SSCC, so we check if it is allowed
                    masterSSCC = barcodeContainerLUID.GS1DataObject.Sscc;
                    goto Step_ValidateSSCC;
                }
                else
                {
                    scannedSSCC = barcodeContainerLUID.FreeBarcode;

                    if (scannedSSCC != null && scannedSSCC.Length == 20 && scannedSSCC.StartsWith("00"))
                    {
                        masterSSCC = scannedSSCC.Substring(2);
                        goto Step_ValidateSSCC;
                    }
                }

                string tempLocation = BarcodeHelper.GetProductIdentification(barcodeContainerLUID);


                if (string.IsNullOrEmpty(tempLocation))
                {
                    initialErrorKey = "MSG_ERROR_BARCODE_DOES_NOT_CONTAIN_SSCC_OR_LOCATION";
                    goto Step_EnterDestinationLocation;
                }
                else
                {
                    //user scanned a location
                    locationCodeChild = tempLocation;
                    goto Step_MakeMove;

                }


            }
            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                
                goto Step_SelectLocation;
            }

            #endregion

            Step_ValidateSSCC:
            #region VALIDATE SSCC
            
            //Scanned SSCC is valid?
            if (!m_sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteIsSSCCValid), masterSSCC))
            {
                initialErrorKey = "MSG_ERROR_INVALID_SSCC";
                goto Step_EnterDestinationLocation;
            }

            
            masterLUID = m_sboProviderService.InvokeMethodWithDbConnection<int?>(false, true, null, null, delegate (PmxDbConnection conn, object[] parameters)
            {
                PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider(conn);
                
                int? tempLUID = null;
                string storLoc = null;
                tempLUID = luidProv.GetCreateMasterSSCC(masterSSCC, out storLoc);
                locationCodeChild = storLoc;
              
                return tempLUID;
            });

            if (!masterLUID.HasValue)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                   "MessageKey", "MSG_ERROR_CANNOT_USE_MASTER_LOG_UNIT",
                       "NoTranslationOfMessageKey", false,
                       "ShowButton", true));
                WaitForMessage();

                goto Step_EnterDestinationLocation;
            }

            if (string.IsNullOrEmpty(locationCodeChild))
            {
                from_ValidateSSCC = true;
                goto Step_IdentifyLocation;
            }

            goto Step_MakeMove;


            ////Scanned SSCC is valid masterSSCC?
            //int? tempMasterLuid = m_sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(ExecuteIsMasterLogUnit), masterSSCC);

            //if (tempMasterLuid.HasValue)
            //{
            //    masterLUID = tempMasterLuid;
            //}
            //else
            //{

            //    initialErrorKey = "MSG_ERROR_CANNOT_USE_MASTER_LOG_UNIT";
            //    goto Step_EnterDestinationLocation;
            //}


            ////check if there are sub sscc         
            //query = BuildQuery.GetProductOnSSCC(masterSSCC, locationCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Inventory, m_sboProviderService.GetDbTool());
            //dsProductsOnSSCC = m_sboProviderService.RunView(false, null, null, query);

            //if (this.GetNumberOfRows(dsProductsOnSSCC) == 0)
            //{
            //    //no subsscc and luid in pmx_lmlu, delete lines
            //    query = BuildQuery.GetLuidsOnMasterLuid(new Collection<int>() { Convert.ToInt32(masterLUID) });
            //    DataSet dsLuids = m_sboProviderService.RunView(true, null, null, query);


            //    //Loop through all LUID's on the master LUID
            //    foreach (DataRow row in dsLuids.Tables[0].Rows)
            //    {
            //        int subLuid = (int)row[PmxLinkMasterLUIDTable.Columns.LUID.NAME];
            //        int masterLuid = (int)row[PmxLinkMasterLUIDTable.Columns.MasterLUID.NAME];

            //         m_sboProviderService.InvokeMethodWithDbConnection<string>(false, true, null, null, delegate (PmxDbConnection conn, object[] parameters)
            //        {
            //           PmxLinkMasterLUIDProvider lmluProv = new PmxLinkMasterLUIDProvider(conn);

            //            CollectionWithDispose<PmxLinkMasterLUID> list = lmluProv.GetLink(masterLuid, subLuid);

            //            if (list == null ||
            //                list.Count == 0)
            //            {
            //                using (PmxLinkMasterLUID link = lmluProv.GetNewBO())
            //                {
            //                    link.MasterLUID = masterLuid;
            //                    link.LUID = subLuid;
            //                    lmluProv.DeleteBO(link);
            //                }
            //            }
            //            return null;
            //        });



            //    }

            //goto Step_MakeMove;
            //}

            //get warehouse
            // whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), dsProductsOnSSCC.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.StorLocCode.NAME] as string);


            #endregion


            Step_IdentifyLocation:

            #region IDENTIFY LOCATION

       
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyLocationScreen),
            DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SCAN_LOCATION",
                "HideDefaultLocation", true,
                "AllowedButtons", new string[] { "MSG_BUTTON_SELECT_LOCATION" }));
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                goto Step_SelectLocation;
            }

            if (msg.Name.EndsWith(".LocationEntered"))
            {
                locationCodeChild = ExtractParameter<string>(msg.Parameters, "locationCode");

                goto Step_SelectLocation;
            }

            goto Step_MakeMove;


            #endregion

            Step_SelectLocation:

            #region SELECT LOCATION

            // Get locations
            query = BuildQuery.GetAllLocations(null , whsInfo.PmxWarehouseCode, deviceID);
            dsLocations = m_sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsLocations) == 0)
            {
                // Inform the user
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                    "MessageKey", "MSG_ERROR_INCORRECT_LOCATION",
                        "NoTranslationOfMessageKey", false,
                        "ShowButton", true));
                WaitForMessage();
                if (from_ValidateSSCC)
                {
                    goto Step_SelectLocation;
                }
                else
                {
                    goto Step_EnterDestinationLocation;
                }
              

            }


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_LOCATION",
                        "LocationDS", dsLocations,
                        "ShowStopButton", false));

            msg = WaitForMessage();
            // Reset latest error
            initialErrorKey = null;

            if (msg.Name.EndsWith(".LocationSelected"))
            {
                locationCodeChild = ExtractParameter<string>(msg.Parameters, "locationCode");

                if (fromStep_IdentifySSCCOrLocation)
                {
                    fromStep_IdentifySSCCOrLocation = false;
                    locationCode = locationCodeChild;
                    goto Step_CheckLocation;
                }
                // Prepare diffs
                goto Step_MakeMove;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
				if (fromStep_IdentifySSCCOrLocation)
				{
					goto Step_IdentifySSCCOrLocation;
				}
                else if (from_ValidateSSCC)
                {
                    goto Step_SelectLocation;
                }
                else
                {
                    goto Step_EnterDestinationLocation;
                }
            }


            #endregion

            Step_MakeMove:

            #region MAKE MOVE

            MoveItemInfo moveInfo = new MoveItemInfo();
            moveInfo.ItemTransactionalInfoKey = itemTransactionalInfoKey;
            moveInfo.DestinationLocationCode = locationCodeChild;
            moveInfo.SourceLocationCode = locationCode;
            moveInfo.SourceLogisticUnitIdentKey = luid;
            moveInfo.DestinationLogisticUnitIdentKey = null;

            moveInfo.DestinationQualityStatusCode = qualityStatusCode;
            moveInfo.ItemCode = itemInfo.ItemCode;
            moveInfo.ItemDescription = itemInfo.CustomItemDescription;
            moveInfo.SourcePmxWhsCode = whsInfo.PmxWarehouseCode;
            moveInfo.SourceQualityStatusCode = qualityStatusCode;
            moveInfo.Uom = itemInfo.SalesUom;
            moveInfo.QuantityPerUom = 1;


            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, true, null, null, delegate (PmxDbConnection conn, object[] parameters)
            {
                PmxShamrockMoveProvider moveProv = new PmxShamrockMoveProvider(conn);
                moveProv.CreateMasterAndChildSSCC(moveInfo, qtyRepack, qtyChild, masterLUID);
                return null;
            });

           

            #endregion

        }




        #endregion



        #region HELPERS
        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }

        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }


        /// <summary>
        /// Executes the get PMX warehouse code.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the is SSCC valid.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static bool ExecuteIsSSCCValid(PmxDbConnection conn, object[] parameters)
        {
            return PmxSSCCProvider.IsValidChecksumSSCC((string)parameters[0]);
        }
        /// <summary>
        /// Executes the is master log unit.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static int? ExecuteIsMasterLogUnit(PmxDbConnection conn, object[] parameters)
        {
            PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider(conn);
            int? luid = null;
            if (!luidProv.CheckIsSSCCMasterLogisticUnit((string)parameters[0], out luid))
            {
                return null;
            }
            return luid;
        }
        /// <summary>
        /// Executes the is location locked.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static bool ExecuteIsLocationLocked(PmxDbConnection conn, object[] parameters)
        {
            PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
            return prov.IsLocationLocked((string)parameters[0]);
        }
        #endregion


        #region BUILD QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {

            /// <summary>
            /// Gets the SSCC on location.
            /// </summary>
            /// <param name="locationCode">The location code.</param>
            /// <returns></returns>
            public static string GetSSCCForLocation(string locationCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, SSCCDataSet.Columns.LUID.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name));

                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));

                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                 SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));



                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
				strBuilder.Append(" IS NOT NULL ");

				if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                }

                return strBuilder.ToString();
            }

            internal static string GetSSCCInfo(string sscc, List<int> listOfLUID)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT DISTINCT {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCDataSet.Columns.LUID.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));

                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));


                strBuilder.AppendFormat("\r\n FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));

                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));



                string operand = "\r\n WHERE ";

                if (!string.IsNullOrEmpty(sscc))
                {
                    strBuilder.Append(operand);
                    strBuilder.AppendFormat(" {0} = {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));
                    operand = "\r\n AND ";
                }


                if (listOfLUID != null && listOfLUID.Count > 0)
                {
                    strBuilder.Append(operand);
                    strBuilder.AppendFormat(" {0} IN ({1})",
                        SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(listOfLUID, CultureInfo.InvariantCulture));
                    operand = "\r\n AND ";
                }

                return strBuilder.ToString();
            }
            /// <summary>
            /// Gets the name of the products from stock order by.
            /// </summary>         
            /// <param name="locationCode">The location code.</param>
            /// <param name="sscc">The SSCC.</param>        
            /// <param name="packagingUnit">The packaging unit.</param> 
            /// <param name="checkSpecificSSCC">if set to <c>true</c> [check specific SSCC].</param>
            /// <returns></returns>
            public static string GetProductsFromStockOrderByName(string locationCode, string sscc, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, bool checkSpecificSSCC,  DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.HasBestBeforeDate.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ManBtchNum.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticUnit.NAME));

                strBuilder.Append(", ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryUom.NAME));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));

                //inner join Product - inventory
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.Columns.ItemCode.NAME));

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemPackagingTypeTable.NAME));
                strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxItemPackagingTypeTable.NAME, PmxItemPackagingTypeTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME));


                string operand = " WHERE ";
              

                if (!string.IsNullOrEmpty(sscc))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }
                else if (checkSpecificSSCC)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                    strBuilder.Append(" IS NULL ");
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(locationCode))
                    {
                        strBuilder.Append(operand);
                        strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                        strBuilder.Append(".");
                        strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                        operand = " AND ";
                    }

              


                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
                operand = " AND ";

             


                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                return strBuilder.ToString();
            }



            /// <summary>
            /// Command for getting all products order by name
            /// </summary>
            /// <param name="sscc">The SSCC.</param>
            /// <param name="locationCode">The location code.</param>
            /// <param name="packagingUnit">The packaging unit.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns>
            /// Command for getting all products order by name
            /// </returns>
            public static string GetProductOnSSCC(string sscc, string locationCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));

                strBuilder.AppendFormat(", " + strBuilder.IFNULL + "({0}, {1}) AS {2} ",
                    SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryUom.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryUom.NAME));


                strBuilder.Append(", SUM( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(strBuilder.AsIdentifier(PmxInventoryTotalTable.Columns.Quantity.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);


                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
              

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryUoMEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                strBuilder.Append(" <> -1 ");


                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);


                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
                strBuilder.Append(" = N'N'");

                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                }


                strBuilder.Append(" GROUP BY ");
             
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CodeBars.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryUom.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME);


                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                return strBuilder.ToString();
            }



            /// <summary>
            /// Gets the batch numbers from stock.
            /// </summary>
            /// <param name="sscc">The SSCC.</param>
            /// <param name="location">The location.</param>
            /// <param name="itemCode">The item code.</param>
            /// <param name="batchNumber">The batch number.</param>
            /// <param name="bestBeforeDate">The best before date.</param>
            /// <param name="qualityStatusCode">The quality status code.</param>
            /// <param name="checkSpecificBatchOrBBD">if set to <c>true</c> [check specific batch or BBD].</param>
            /// <param name="checkSpecificSSCC">if set to <c>true</c> [check specific SSCC].</param>
            /// <param name="allowedQualityStatus">The allowed quality status.</param>
            /// <param name="restLocationCode">The rest location code.</param>
            /// <returns></returns>
            public static string GetBatchNumbersFromStock(string sscc, string location, string itemCode, string batchNumber, DateTime? bestBeforeDate, string qualityStatusCode, bool checkSpecificBatchOrBBD, bool checkSpecificSSCC)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, BatchNumberDataSet.Columns.BATCH_NUMBER.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, BatchNumberDataSet.Columns.BATCH_NUMBER2.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, BatchNumberDataSet.Columns.BEST_BEFORE_DATE.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, BatchNumberDataSet.Columns.QUALITY_STATUS_CODE.Name));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, BatchNumberDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));
                strBuilder.Append(", SUM( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(BatchNumberDataSet.Columns.QUANTITY.Name));


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);



                string operand = " WHERE ";

                if (!string.IsNullOrEmpty(sscc))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }
                else if (checkSpecificSSCC)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                    strBuilder.Append(" IS NULL ");
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(qualityStatusCode))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(qualityStatusCode, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }


                if (!string.IsNullOrEmpty(location))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                  
                  
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(location, CultureInfo.InvariantCulture));
                
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(itemCode))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(batchNumber))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }
                else if (checkSpecificBatchOrBBD)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                    strBuilder.Append(" IS NULL ");
                    operand = " AND ";

                }

                if (bestBeforeDate.HasValue)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(bestBeforeDate.Value, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }
                else if (checkSpecificBatchOrBBD)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
                    strBuilder.Append(" IS NULL ");
                    operand = " AND ";

                }

              

                strBuilder.Append(" GROUP BY ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);




                return strBuilder.ToString();
            }



            /// <summary>
            /// Gets all the active location.
            /// </summary>
            /// <param name="locationCode">The location code.</param>
            /// <param name="warehouseCode">The pmx warehouseCode</param>
            /// <param name="deviceID">The device ID.</param>
            /// <returns>
            /// The sql statement to get all the active location
            /// </returns>
            public static string GetAllLocations(string locationCode, string pmxWarehouseCode, string deviceID)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxUserTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.LockedBy.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxUserTable.NAME, PmxUserTable.Columns.InternalKey.NAME);

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

                }

                if (!String.IsNullOrEmpty(pmxWarehouseCode) || !String.IsNullOrEmpty(deviceID))
                {
                    strBuilder.Append(" AND ( ");
                    if (!String.IsNullOrEmpty(pmxWarehouseCode))
                    {
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pmxWarehouseCode, CultureInfo.InvariantCulture));
                    }
                    if (!String.IsNullOrEmpty(deviceID))
                    {
                        if (!String.IsNullOrEmpty(pmxWarehouseCode))
                            strBuilder.Append(" OR ");

                        strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                            SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                            SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                    }
                    strBuilder.Append(" )");
                }

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));

                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the luids configuration master luid.
            /// </summary>
            /// <param name="masterLuids">The master luids.</param>
            /// <param name="cultureInfo">The culture information.</param>
            /// <returns></returns>
            public static string GetLuidsOnMasterLuid(Collection<int> masterLuids)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkMasterLUIDTable.NAME, PmxLinkMasterLUIDTable.Columns.MasterLUID.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkMasterLUIDTable.NAME, PmxLinkMasterLUIDTable.Columns.LUID.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinkMasterLUIDTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxLinkMasterLUIDTable.NAME, PmxLinkMasterLUIDTable.Columns.LUID.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLinkMasterLUIDTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLinkMasterLUIDTable.Columns.MasterLUID.NAME));
                strBuilder.Append(" IN ( ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(masterLuids, CultureInfo.InvariantCulture));
                strBuilder.Append(" ) ");
                return strBuilder.ToString();
            }
            /// <summary>
            /// Gets the name of the products from stock order by.
            /// </summary>
            /// <param name="gtin">The gtin.</param>
            /// <param name="itemCode">The item code.</param>
            /// <param name="locationCode">The location code.</param>
            /// <param name="sscc">The SSCC.</param>
            /// <param name="forceSpecific">if set to <c>true</c> [force specific].</param>
            /// <param name="luid">The luid.</param>
            /// <param name="packagingUnit">The packaging unit.</param>
            /// <param name="restLocationCode">The rest location code.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            public static string GetProductsFromStockOrderByName( string locationCode, string sscc,  int? luid, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append(
                    "SELECT DISTINCT ", strBuilder.AsIdentifierWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name),

                    ", ", strBuilder.AsIdentifierWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name),

                    ", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

                strBuilder.Append(
                    ", ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.HasBestBeforeDate.NAME),

                    ", ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ManBtchNum.NAME),

                    ", ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticUnit.NAME),

                    ", ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryUom.NAME),


                    "\r\n FROM ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME),

                    //inner join Product - inventory
                    "\r\n INNER JOIN ", strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME),
                                " ON ", strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME),
                                 " = ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME),

                    "\r\n LEFT JOIN ", strBuilder.AsIdentifier(PmxItemPackagingTypeTable.NAME),
                               " ON ", strBuilder.AsIdentifier(PmxItemPackagingTypeTable.NAME, PmxItemPackagingTypeTable.Columns.ItemCode.NAME),
                                " = ", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME));


                string operand = " WHERE ";
              

                if (!string.IsNullOrEmpty(sscc))
                {
                    strBuilder.Append(operand, strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME),
                        " = ", strBuilder.AsString(sscc));
                    operand = " AND ";
                }
               

                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(operand, strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));                  
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));                   
                    operand = " AND ";
                }

                if (luid.HasValue)
                {
                    strBuilder.AppendFormat(" {0} {1}.{2}  = {3} ", operand, SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME), SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME), SqlCommandHelper.FormatAndEscapeColumnValue(luid.Value, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }



                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ", strBuilder.IFNULL, "(", strBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UseForTimeRegistration.NAME));
                strBuilder.AppendFormat(", {0} ) = {0} ",
                SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

              
                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                return strBuilder.ToString();
            }

        }

        #endregion
    }

}